﻿using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Validation.Exceptions;
using Exam.Infrastructure.Validation.Extensions;
using Exam.Infrastructure.Validation.Validation;
using FluentValidation;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Interface;
using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Exceptions;

namespace Exam.Repository.Sqlsugar.Repositories
{
    public class ExamRepository<TEntity> : BaseRepository<TEntity>, IExamRepository<TEntity, HotlineDbContext> where TEntity : class, IEntity<string>, IHasCreationTime, IDataPermission, new()
    {
        public ExamRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider) : base(uow, dataPermissionFilterBuilder, serviceProvider)
        {
            _uow = uow;
        }

        private readonly ISugarUnitOfWork<HotlineDbContext> _uow;
        public ISugarUnitOfWork<HotlineDbContext> UOW
        {
            get
            {
                return _uow;
            }
        }

        public AbstractValidator<TEntity> Validator { get; set; }

        #region public method
        public async Task AddWithValidateAsync(TEntity entity, CancellationToken cancellationToken)
        {
            await DoValidationAsync(entity,ValidatorTypeConstants.Create);

            await base.AddAsync(entity, cancellationToken);
        }

        public async Task AddWithValidateAsync(List<TEntity> entities, CancellationToken cancellationToken)
        {
            await DoValidationAsync(entities, ValidatorTypeConstants.Create);

            await base.AddRangeAsync(entities, cancellationToken);
        }

        public async Task DeleteWithValidateAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            if (entityQueryRequest.Ids != null && entityQueryRequest.Ids.Any())
            {
                var entities = await base.QueryAsync(x => entityQueryRequest.Ids.Contains(x.Id));

                await base.RemoveRangeAsync(entities, cancellationToken);
            }
            else
            {
                await base.RemoveAsync(entityQueryRequest.Id);
            }
        }

        public async Task UpdateWithValidateAsync(TEntity entity, CancellationToken cancellationToken)
        {
            await DoValidationAsync(entity, ValidatorTypeConstants.Modify);

            await base.UpdateAsync(entity, cancellationToken);
        }

        public async Task UpdateWithValidateAsync(List<TEntity> entities, CancellationToken cancellationToken)
        {
            await DoValidationAsync(entities, ValidatorTypeConstants.Modify);

            await base.UpdateRangeAsync(entities, cancellationToken);
        }
        #endregion

        #region protected method
        /// <summary>
        /// 验证
        /// </summary>
        /// <param name="entity"></param>
        /// <exception cref="NotImplementedException"></exception>
        protected async Task DoValidationAsync(TEntity entity, string validatorType)
        {
            if (Validator == null)
                throw new NotImplementedException("验证器未初始化");

            var validatorResult = await Validator.DoValidateAsync(entity, validatorType);
            if (!validatorResult.IsValid)
            {
                var errorMessages = validatorResult.ErrorItems.Select(x => x.ErrorMessage);
                throw new UserFriendlyException(string.Join(";", errorMessages));
            }
        }

        /// <summary>
        /// 验证
        /// </summary>
        /// <param name="entity"></param>
        /// <exception cref="NotImplementedException"></exception>
        protected async Task DoValidationAsync(List<TEntity> entities, string validatorType)
        {
            if (Validator == null)
                throw new NotImplementedException("验证器未初始化");

            var domainException = new DomainException();
            foreach (var entity in entities)
            {
                var validatorResult = await Validator.DoValidateAsync(entity, validatorType);
                if (!validatorResult.IsValid)
                {
                    domainException.AddErrors(validatorResult);
                }
            }
            if (domainException.ValidationErrors.ErrorItems.Any())
            {
                var errorMessages = domainException.ValidationErrors.ErrorItems.Select(x => x.ErrorMessage);
                throw new UserFriendlyException(string.Join(";", errorMessages));
            }
        }
        #endregion
    }
}
